
! Last change : Ino&Hans 8 December 2014
!               Gabor    22 January 2015
! Basic instructions for running GRain from the command line 
!     
!1. After starting the program, the user is asked to enter 
!   the name of the input pedigree file, which has to be saved 
!   in the same directory as the GRain program.
!2. Following, the user should decide what calculations are going 
!   to be performed i.e. to choose output files to be created;
!	1 = output file with prefix tot_,
!	2 = output file with prefix F_,
!	3 = output file with prefix Fa_BAL_,
!	4 = output file with prefix AHC_, 
!	5 = output file with prefix Fa_KAL_,
!	0= all possible output files are created, all output file printed.
!
!3a. If the user decides to calculate all non-partial coefficients (option number 1)
!    the program is proceeding directly to the number of replications.
!
!or
!
!3b. If the user decides to calculate partial coefficients (options 0, 2, 3, 4 and 5),
!    the total number of founders to be analyzed and the name of the list file with 
!    founders/ancestors to be analyzed needs to be provided. The file
!    has to be saved in the same directory as the GRain program. 
!
!4. The user should enter the number of replications to be executed, i.e. the number of 
!    gene drops, to be executed (there is no limit for the number entered).
!5. The program runs and quits by pressing the ENTER key.

!     
!	program GRain
!******** programme to calculate inbreeding coefficients, 3 different
!******** ancestral inbreeding coefficients and corresponding partial
!******** inbreeding coefficients
!******** by R. BAUMUNG et al. (6/06)

!** simulation of segregation 
!** file 5
!**       name of pedigree file
!**       nb of replicates,
!** file 1 (pedigree) : same as in the other program
!**   ie individual, sire, dam, separated by blank

!========================================================================
!   Pedigree with animal, sire and dam,
!   USE PROGRAMME JUST FOR CHECKED PEDIGREES
!========================================================================

!**
!**  Compiler options
!**
!** gfortran -static
!**

      MODULE COMMON_DECL

        INTEGER(KIND = 4), PARAMETER :: Char_Len = 128
        CHARACTER(LEN = 20), PARAMETER :: Summary = 'Summary.txt'

        INTEGER(Kind = 4), DIMENSION (:), ALLOCATABLE      :: Prc
        INTEGER(Kind = 4), DIMENSION (:), ALLOCATABLE      :: Mrc
        INTEGER(Kind = 4), DIMENSION (:), ALLOCATABLE      :: Ngen
        INTEGER(Kind = 4), DIMENSION (:), ALLOCATABLE      :: Ord
        INTEGER(Kind = 4), DIMENSION (:), ALLOCATABLE      :: Lf
        INTEGER(Kind = 4), DIMENSION (:), ALLOCATABLE      :: Listf
        INTEGER(Kind = 4), DIMENSION (:,:), ALLOCATABLE    :: All
        INTEGER(Kind = 4), DIMENSION (:,:), ALLOCATABLE    :: Al
        INTEGER(Kind = 4), DIMENSION (:,:), ALLOCATABLE    :: Flag
        INTEGER(Kind = 4), DIMENSION (:,:), ALLOCATABLE    :: Allp
        INTEGER(Kind = 4), DIMENSION (:,:), ALLOCATABLE    :: Alp
        INTEGER(Kind = 4), DIMENSION (:,:), ALLOCATABLE    :: Flags
        INTEGER(Kind = 4), DIMENSION (:,:,:), ALLOCATABLE  :: Flag1
        INTEGER(Kind = 4), DIMENSION (:,:,:), ALLOCATABLE  :: Flags1

        REAL(Kind = 8), DIMENSION (:), ALLOCATABLE         :: Anc
        REAL(Kind = 8), DIMENSION (:), ALLOCATABLE         :: Inb
        REAL(Kind = 8), DIMENSION (:), ALLOCATABLE         :: Ancs
        REAL(Kind = 8), DIMENSION (:), ALLOCATABLE         :: Anck
        REAL(Kind = 8), DIMENSION (:), ALLOCATABLE         :: Fn

        REAL(Kind = 8), DIMENSION (:,:), ALLOCATABLE       :: Inbp
        REAL(Kind = 8), DIMENSION (:,:), ALLOCATABLE       :: Ancp
        REAL(Kind = 8), DIMENSION (:,:), ALLOCATABLE       :: Anckp
        REAL(Kind = 8), DIMENSION (:,:), ALLOCATABLE       :: Ancsp

        INTEGER(Kind = 4)   :: I, J, K, N, NL, NBIT, IP, IM
        INTEGER(Kind = 4)   :: II, NOUT, L, NALL, NSIM, M, KK, JJ, Write_NFF
        INTEGER(Kind = 4)   :: NT, NF, NFF, IOS, STATUS, Nt_Default, Expr_Default
        REAL(Kind = 4)      :: X
        LOGICAL             :: T

        CHARACTER(LEN = Char_Len)   :: Str, Strout, Strout1, Strout2, Strout3
        CHARACTER(LEN = Char_Len)   :: Strout4, Fancest
! inserted April 21, 2012 (JS)
        CHARACTER(LEN = Char_Len)   :: yesno

! end insert

        REAL(Kind = 8)      :: Start_Time, End_Time
        CHARACTER(LEN = 24) :: Start_Date_Time, End_Date_Time
        INTEGER(Kind = 4)   :: Hour, Minute, Sec
        REAL(Kind = 8)      :: Elapsed_Time

      END MODULE COMMON_DECL
!
!     Program GRain
!
      USE COMMON_DECL

      IMPLICIT NONE

      INTEGER(Kind = 8)  :: Array_Size

      CALL Cpu_Time(Start_Time)
      CALL FDATE(Start_Date_Time)
      PRINT *, 'Start Date: ', Start_Date_Time
      PRINT *, '************************************'

!     Read Input Data

      PRINT *, ''
      PRINT *, ' Name of pedigree file: '
      READ(*,'(A)') Str

      OPEN (10, FILE=Str, STATUS='OLD', IOSTAT=IOS)
      IF (IOS .NE. 0) THEN
        PRINT *, ''
        PRINT *, '!!! ERROR: Pedigree file is missing!'
        PRINT *, 'Name of Pedigree files: ', Str
        PRINT *, ''
        CALL Press_Enter()
      END IF

      READ(10, *, IOSTAT=IOS) I, IP, IM

      Nt_Default = 0
      Expr_Default = 0
      DO WHILE (IOS == 0)
		IF (I.EQ.0) EXIT
        Nt_Default = Nt_Default + 1
        IF (IP.LT.0) IP = 0
        IF (IM.LT.0) IM = 0

        IF ((IP.EQ.0.and.IM.GT.0).OR.(IP.GT.0.AND.IM.EQ.0)) THEN
			Expr_Default = Expr_Default + 1
		END IF

        READ(10, *, IOSTAT=IOS) I, IP, IM
      END DO
      CLOSE(10)

      PRINT *,''
      IF (Nt_Default == 0) THEN
        PRINT *,''
      	PRINT *,' !!! ERROR: Pedigree file is empty!'
        PRINT *,''
        CALL Press_Enter()
      END IF

! inserted on April 21, 2012 (JS)

      PRINT *, ''
      PRINT *, ' Which output file(s) do you want to create?'
      PRINT *, ' ========================================='
      PRINT *, ' For all total coefficients press 1'
      PRINT *, ' For F plus partial inbreeding coefficients press 2'
      PRINT *, ' For Fa_ballou plus partial inbreeding coefficients press 3'
      PRINT *, ' For AHC plus partial inbreeding coefficients press 4'
      PRINT *, ' For Fa_Kalinowski plus partial inbreeding coefficients press 5'
      PRINT *, ' For all five output files press 0'
      PRINT *, ' ========================================='
      PRINT *, ''
      PRINT *, ' Provide option: '
      READ (*,'(I10)') NOUT
      IF (NOUT.LE.0.OR.NOUT.GT.5) NOUT = 0
      
      IF(NOUT.EQ.0.OR.NOUT.GT.1) THEN
      
      	PRINT *, ' Contributions of the how many founders in total shall be analysed (NF>=1): '
      	READ(*,'(I5)')  NF
      	IF (NF.LE.0) NF = 1

      	PRINT *, ''
      	PRINT *, ' Name of file of ancestors to be analysed : '
      	READ (5,*) Fancest

      	IF (Fancest.NE.'no') THEN
			OPEN (10, FILE=Fancest, STATUS='OLD', IOSTAT=IOS)
				IF (IOS .NE. 0) THEN
					PRINT *, ''
					PRINT *, '!!! ERROR: Ancestor file is missing!'
					PRINT *, 'Name of Ancestor files: ', Fancest
					PRINT *, ''
					CALL Press_Enter()
				END IF
			CLOSE (10)
        END IF

      	PRINT *, ''
      	IF (Fancest.EQ.'no') THEN
        	PRINT *,' Contributions of NF = ', NF, ' first founders analysed'
      	ELSE
        	PRINT *,' Contributions of ancestors in the file "',TRIM(Fancest), '" analysed first'
        	PRINT *,' as well as some complentary founders (NF in total)'
      	END IF
      END IF

      PRINT *, ''                             
      PRINT *, ' Number of replicates: '
      READ (*,'(I10)') NSIM
      IF (NSIM.LT.1) NSIM = 1
          
      NT = Nt_Default + Expr_Default
      PRINT *, ''
      PRINT *, '********************************************************'
      PRINT *, ''
      PRINT *, ' Number of Pedigree records : ', Nt_Default
      PRINT *, ''
      PRINT *, ' Number of extended Pedigree records : ', Expr_Default
      PRINT *, ''
      PRINT *, ' Total Pedigree records (Pedigree + Extended): ', NT

!     Allocate memory for the current (input) parameter

!     Minimum array memory size
!     1*nf*4; 5*nt*4; 6*2*nt*4; 5*nt*8; 4*nt*nf*8; 2*2*nt*nf*8

      Array_Size = 4*NF + 108*NT + 64*NT*NF

      PRINT *, ''
      PRINT *, ' Needs minimum memory size: ', INT(Array_Size/10**6)+1, ' MByte'
      PRINT *, ''

      ALLOCATE(Listf(NF), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Prc(NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Mrc(NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Ngen(NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Ord(NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Lf(NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Anc(NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Inb(NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Ancs(NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Anck(NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Fn(NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(All(2,NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Al(2,NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Flag(2,NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Allp(2,NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Alp(2,NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Flags(2,NT), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Inbp(NT,NF), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Ancp(NT,NF), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Anckp(NT,NF), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Ancsp(NT,NF), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Flag1(2,NT,NF), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      ALLOCATE(Flags1(2,NT,NF), STAT = STATUS)
        IF (STATUS > 0) CALL Press_Enter_Memory()

      PRINT *, ' Allocation of memory is OK!'
      PRINT *, ''
      PRINT *, '********************************************************'
      PRINT *, ''

!   read in from pedigree file str: animal, sire, dam

      OPEN (10, FILE=STR, STATUS='OLD', IOSTAT=IOS)
      IF (IOS .NE. 0) THEN
        PRINT *, ''
        PRINT *, '!!! ERROR: Pedigree file is missing!'
        PRINT *, 'Name of Pedigree files: ', Str
        PRINT *, ''
        CALL Press_Enter()
      END IF

      READ(10, *, IOSTAT=IOS) I, Ip, Im

      NL = 0
      DO WHILE (IOS == 0)
		IF (I.EQ.0) EXIT
        NL = NL + 1
		IF (NL.NE.I) THEN
			PRINT *, ''
			PRINT *, '!!! ERROR: Pedigree file in a wrong order'
			PRINT *, ''
			CALL Press_Enter()
        END IF
        Prc(NL) = IP
        Mrc(NL) = IM
        IF (IP.LT.0) Prc(NL) = 0
        IF (IM.LT.0) Mrc(NL) = 0

        READ(10, *, IOSTAT=IOS) I, IP, IM
      END DO

      CLOSE(10)

! extension of pedigree for animals with one known parent

      KK = NL
      DO I = 1, NL
        IF (Prc(I).EQ.0.and.Mrc(I).GT.0) THEN
			KK = KK + 1
			Prc(KK)=0
			Mrc(KK)=0
			Prc(I) = KK
        ELSE IF (Prc(I).GT.0.AND.Mrc(I).EQ.0) THEN
			KK = KK + 1
			Prc(KK) = 0
			Mrc(KK) = 0
			Mrc(I) = KK
		END IF
      END DO

      NL = KK

      DO I = 1, NL
        Lf(I) = 0
      END DO

! read the file of analysed founders

      NFF = 0
      IF (Fancest.ne.'no') THEN
		OPEN (10, FILE=Fancest, STATUS='OLD', IOSTAT=IOS)

        READ(10, *, IOSTAT=IOS) I
        DO WHILE (IOS == 0)
			IF (I.LT.1.OR.I.GT.NL) THEN
				PRINT *,'NL = ', NL,' Ancestor ID= ', I
				PRINT *,''
				PRINT *,'!!! ERROR: Ancestor read not in the pedigree file'
				PRINT *,''
				CALL Press_Enter()
			END IF
			NFF = NFF + 1
			Listf(NFF) = I
			Prc(I)= 0  ! ancestors become founders
			Mrc(I)= 0
			Lf(I) = NFF

			READ(10, *, IOSTAT=IOS) I
        END DO
        CLOSE (10)

        Write_NFF = NFF
        PRINT *, ' Number of ancestors requested for analysis by the user: ', NFF
        IF (NF.LT.NFF) PRINT *, ' lower than total number to be analysed: ', NF

      END IF

! alleles for base animals

      NALL = 0
      DO I = 1, NL
        All(1,I) = 0
        All(2,I) = 0
        IF (Prc(I).EQ.0)  THEN
          NALL = NALL + 1
          All(1,I) = NALL
          Allp(1,I) = I
        END IF
        IF (Mrc(I).EQ.0)  THEN
          NALL = NALL + 1
          All(2,I) = NALL
          Allp(2,I) = I
        END IF
        IF (Prc(I).EQ.0.OR.Mrc(I).EQ.0)  THEN
			IF (Lf(I).EQ.0) THEN
				NFF = NFF + 1
				IF (NFF.LE.NF) THEN
					Listf(NFF) = I
					Lf(I) = NFF
				END IF
			END IF
        END IF
      END DO

!---only the number of founders given in NF is considered

      IF (NFF.GT.NF) NFF = NF

      PRINT *, ''
      PRINT *, ' NB of alleles in segregation : ', NALL

!---create name of output pedigree files
      Strout  = 'tot_'//Str
      Strout1 = 'F_'//Str
      Strout2 = 'Fa_bal_'//Str
      Strout3 = 'AHC_'//Str
      Strout4 = 'Fa_kal_'//Str

!*****opening one of the chosen or all output files
      IF (NOUT.EQ.0.OR.NOUT.eq.1) THEN
        OPEN (2,FILE=strout, FORM='formatted')
      endif
      IF (NOUT.EQ.0.OR.NOUT.eq.2) THEN
        OPEN (3,FILE=strout1, FORM='formatted')
      endif
      IF (NOUT.EQ.0.OR.NOUT.eq.3) THEN
        OPEN(4,FILE=strout2, FORM='formatted')
      endif
      IF (NOUT.EQ.0.OR.NOUT.eq.4) THEN
        OPEN(7,FILE=strout3, FORM='formatted')
      endif
      IF (NOUT.EQ.0.OR.NOUT.eq.5) THEN
        OPEN(8,FILE=strout4, FORM='formatted')
      endif

      call random_seed

! ordering of animals - ngen=0 for founders, >0 for others
      DO I = 1, NL
        Ord(I)=0
      END DO

      K = 0
      DO I = 1, NL
        Ngen(I) = -1
        IF (Prc(I).EQ.0 .and.Mrc(I).EQ.0)  THEN
            Ngen(I) = 0
            K = K + 1
            Ord(K) = I
        END IF
      END DO

      NBIT = 0
      DO WHILE (K.LT.NL.AND.NBIT.LE.20)
        NBIT = NBIT + 1
        J = 0
        DO I = 1, NL
          IF (Ngen(I).LT.0) THEN
            IP = Prc(I)
            iM = Mrc(I)
            T = .true.
            IF (IP.NE.0.and.Ngen(IP).LT.0) T = .false.
            IF (IM.NE.0.and.Ngen(IP).LT.0) T = .false.
            IF (T) THEN
              IF (IP.EQ.0) THEN
                II = 0
              ELSE
                II = Ngen(IP)
              END IF
              IF (IM.NE.0) II = MAX(II,Ngen(IM))
              Ngen(I) = II + 1
              K = K + 1
              Ord(K) = I
              J = J + 1
            END IF
          END IF
        END DO
      END DO

!  searching for errors in pedigrees

      J = 0
      IF (K.NE.NL) THEN
        DO I = 1, NL
          IF (Ngen(I).LT.0) THEN
            J = J + 1
            IF (J.LT.100) WRITE(*,*) I, Prc(I), Mrc(I)
            Prc(I) = 0
            Mrc(I) = 0
            K = K + 1
            Ord(K)=I
            Ngen(I)=0
          END IF
        END DO
      END IF
      IF (J.GT.0) PRINT *, J, ' errors in pedigree - Impossible parents information is deleted'

!   setting vectors for all inbreeding coefficients to zero

      DO I = 1, NL
!        total inbreeding coefficients
         Anc(I) = 0.
         Inb(I) = 0.
         Ancs(I) = 0.
         Anck(I) = 0.
         Fn(I) = 0.

!        partial inbreeding coefficients except for fac
         DO J = 1, NFF
           Ancp(I,J) = 0.
           Inbp(I,J) = 0.
           Anckp(I,J) = 0.
         END DO
      END DO
      PRINT *, ' Total Number of founders analysed for contributions : ',nff

!    starting the simulation of gene drop from now on

      PRINT *, ''
      PRINT *, ' Working simulation ...'
      DO II = 1, NSIM
      	IF (MOD(II,500).EQ.0) PRINT *, ' Working simulation record: ', II

      	DO I = 1, NL
          al(1,i)=all(1,i)
          al(2,i)=all(2,i)
          alp(1,i)=allp(1,i)
          alp(2,i)=allp(2,i)
          flag(1,i)=0
          flag(2,i)=0
          flags(1,i)=0
          flags(2,i)=0

          DO J = 1, NFF
            flag1(1,i,j)=0
            flag1(2,i,j)=0
            flags1(1,i,j)=0
            flags1(2,i,j)=0
          END DO
        END DO

      	DO KK = 1, NL   !  beginning of individuals
          i=ord(kk)
          IF (prc(i).gt.0) THEN
            call random_number(x)
            k=1
            IF (x.gt..5d0) k=2

            al(1,i)=al(k,prc(i))
            alp(1,i)=alp(k,prc(i))
            IF (flag(k,prc(i)).ge.1) THEN
              flag(1,i)=1
              flags(1,i)=flags(k,prc(i))
            END IF
            DO  J = 1, NFF
!***    keep information on past IBD
              IF (flag1(k,prc(i),j).ge.1)  flag1(1,i,j)=1
              IF (flags1(k,prc(i),j).ge.1) flags1(1,i,j)=flags1(k,prc(i),j)
            END DO
          END IF

          IF (mrc(i).gt.0) THEN
            call random_number(x)
            k=1
            IF (x.gt..5d0) k=2

            al(2,i)=al(k,mrc(i))
            alp(2,i)=alp(k,mrc(i))
            IF (flag(k,mrc(i)).ge.1) THEN
               flag(2,i)=1
               flags(2,i)=flags(k,mrc(i))
            END IF
            DO J = 1, NFF
              IF (flag1(k,mrc(i),j).ge.1)  flag1(2,i,j)=1
              IF (flags1(k,mrc(i),j).ge.1) flags1(2,i,j)=flags1(k,mrc(i),j)
            END DO
          END IF

!***   sum up number flagged alleles over repetitions
          anc(i)=anc(i)   + flag(1,i)  + flag(2,i)
          ancs(i)=ancs(i) + flags(1,i) + flags(2,i)

          DO J = 1, NFF
			jj=listf(j)
            IF (alp(1,i).eq.jj) THEN
              ancp(i,j) =ancp(i,j)  + flag1(1,i,j)
              ancsp(i,j)=ancsp(i,j) + flags1(1,i,j)
              IF (al(1,i).eq.al(2,i)) THEN
                anckp(i,j)=anckp(i,j)+ MAX(flag1(1,i,j),flag1(2,i,j))
              END IF
            END IF
            IF (alp(2,i).eq.jj) THEN
              ancp(i,j) =ancp(i,j)  + flag1(2,i,j)
              ancsp(i,j)=ancsp(i,j) + flags1(2,i,j)
              IF (al(1,i).eq.al(2,i)) THEN
                anckp(i,j)=anckp(i,j)+ MAX(flag1(2,i,j),flag1(1,i,j))
              END IF
            END IF
          END DO

          IF (al(1,i).eq.al(2,i)) THEN
            inb(i)=inb(i)+1
            anck(i)=anck(i) + Max(flag(1,i),flag(2,i))
            fn(i)=(2*inb(i) - anck(i))
!***    flag alleles which are IBD
            flag(1,i)=1
            flag(2,i)=1
            flags(1,i)=flags(1,i)+1
            flags(2,i)=flags(2,i)+1

            DO J = 1, NFF
			  jj=listf(j)
              IF (alp(1,i).eq.jj) THEN
                flag1(1,i,j)=flag1(1,i,j)+1
                flag1(2,i,j)=flag1(2,i,j)+1
                flags1(1,i,j)=flags1(1,i,j)+1
                flags1(2,i,j)=flags1(2,i,j)+1
                inbp(i,j)=inbp(i,j)+1
              END IF
            END DO
          END IF

        END DO ! end of individual

      END DO  ! end of replicate
!*** END OF REPLICATE

! writing pedigree, ancestral inbreeding coefficient
! and inbreeding coefficient into output file

      DO K = 1, NL
        I = Ord(K)
        Anc(I) = Anc(I)/REAL(2*NSIM)     ! ancestral Ballou (1997)
        Inb(I) = Inb(I)/REAL(NSIM)       ! classical inbreeding coeff.
		Ancs(I) = Ancs(I)/REAL(2*NSIM)   ! ancestral Baumung et al. (2006)

        Fn(I) = Fn(I)/REAL(2*NSIM)       ! new inbreeding Kalinowski et al. (2000)
        Anck(I) = Anck(I)/REAL(NSIM)     ! ancestral Kalinowski et al. (2000)

        DO J = 1, NFF
          Inbp(I,J) = Inbp(I,J)/REAL(NSIM)       ! partial classical
          Ancp(I,J) = Ancp(I,J)/REAL(2*NSIM)     ! partial Ballou
          Ancsp(I,J) = Ancsp(I,J)/REAL(2*NSIM)   ! partial Baumung et al.
          Anckp(I,J) = Anckp(I,J)/REAL(2*NSIM)   ! partial Kalinowski et al.
        END DO
      END DO
      
      IF (NOUT.EQ.0.OR.NOUT.EQ.2) WRITE(3 ,'(32x,400i7 )')   (Listf(I), I =1, NFF)
      IF (NOUT.EQ.0.OR.NOUT.EQ.3) WRITE(4 ,'(32x,400i7 )')   (Listf(I), I =1, NFF)
      IF (NOUT.EQ.0.OR.NOUT.EQ.4) WRITE(7 ,'(32x,400i7 )')   (Listf(I), I =1, NFF)
      IF (NOUT.EQ.0.OR.NOUT.EQ.5) WRITE(8 ,'(32x,400i7 )')   (Listf(I), I =1, NFF)
      IF (NOUT.EQ.0.OR.NOUT.EQ.1) THEN
          WRITE (2,*) ' animal  sire    dam     f-classical fa-Ballou  AHC  fa-Kalinowski'
      END IF

      DO K = 1, NL
          I = Ord(K)
          IF (NOUT.EQ.0.OR.NOUT.EQ.1) THEN
            WRITE(2 ,'(i8,i8,i8,1x,f6.4, 1x, f6.4,  &
     &           1x, f6.4,1x,f6.4)')  &
     &       I, Prc(I), Mrc(I), Inb(I), Anc(I), Ancs(I), Anck(I)
          END IF

          IF (NOUT.EQ.0.OR.NOUT.EQ.2) THEN
            WRITE(3 ,'(i8,i8,i8,1x,f6.4,1x,225(f6.4,1x) )')   &
     &      I, Prc(I), Mrc(I), Inb(I), (Inbp(I,J), J = 1, NFF)
          END IF

          IF (NOUT.EQ.0.OR.NOUT.EQ.3) THEN
            WRITE(4 ,'(i8,i8,i8,1x,f6.4,1x,225(f6.4,1x) )')   &
     &       I, Prc(I), Mrc(I), Anc(I), (Ancp(I,J), J = 1, NFF)
          END IF

          IF (NOUT.EQ.0.OR.NOUT.EQ.4) THEN
            WRITE(7 ,'(i8,i8,i8,1x,f6.4,1x,225(f6.4,1x) )')    &
     &       I, Prc(I), Mrc(I), Ancs(I), (Ancsp(I,J), J = 1, NFF)
          END IF

          IF (NOUT.EQ.0.OR.NOUT.EQ.5) THEN
            WRITE(8 ,'(i8,i8,i8,1x,f6.4,1x,225(f6.4,1x) )')    &
     &          I, Prc(I), Mrc(I), Anck(I), (Anckp(I,J), J = 1, NFF)
          END IF

      END DO

      CALL Cpu_Time(End_Time)
      Elapsed_Time  = NINT(End_Time  - Start_Time)
      Hour   = Elapsed_Time / 3600
      Minute = (Elapsed_Time - Hour*3600) / 60
      Sec = Elapsed_Time - Hour*3600 - Minute*60
      PRINT *, ''
      WRITE (*,'('' Elapsed CPU time: '',I3.2,'':'',I2.2,'':'',I2.2)') Hour, Minute, Sec

      CALL FDATE(End_Date_Time)
      PRINT *, ''
      PRINT *, 'End Date: ', End_Date_Time
      PRINT *, '**********************************'
      PRINT *, ''

      OPEN (10, FILE=Summary, STATUS='UNKNOWN')
      WRITE (10, '('' '')')
      WRITE (10, '(''Start Date: '',A)') Start_Date_Time
      WRITE (10, '(''************************************'')')
      WRITE (10, '('' '')')
      WRITE (10, '(''Name of pedigree files: '',A)') TRIM(Str)
      WRITE (10, '('' '')')

      IF (NOUT.NE.1) THEN
      	WRITE (10, '(''Contributions of the how many founders in total shall be analysed: '',I10)') NF
      	WRITE (10, '('' '')')
      	WRITE (10, '(''Name of file of ancestors to be analysed: '',A)') TRIM(Fancest)
      	WRITE (10, '('' '')')
      	IF (Fancest.EQ.'no') THEN
           WRITE (10, '(''Contributions of NF = '', I10, '' first founders analysed'')') NF
      	ELSE
           WRITE (10, '(''Contributions of ancestors in the file '',A, '' analysed first'')') TRIM(Fancest)
           WRITE (10, '(''as well as some complentary founders (NF in total)'')')
      	END IF
      	WRITE (10, '('' '')')
      END IF

      WRITE (10, '(''Number of replicates: '', I10)') NSIM
      WRITE (10, '('' '')')
      WRITE (10, '(''********************************************************'')')
      WRITE (10, '('' '')')
      WRITE (10, '(''Number of Pedigree records: '',I10)') Nt_Default
      WRITE (10, '('' '')')
      WRITE (10, '(''Number of extended Pedigree records: '',I10)') Expr_Default
      WRITE (10, '('' '')')
      WRITE (10, '(''Total Pedigree records (Pedigree + Extended): '',I10)') NT
      WRITE (10, '('' '')')
      WRITE (10, '(''Needs minimum memory size: '',I10, '' MByte'')') INT(Array_Size/10**6)+1
      WRITE (10, '('' '')')
      WRITE (10, '(''Allocation of memory is OK!'')')
      WRITE (10, '('' '')')
      WRITE (10, '(''********************************************************'')')

      IF (Fancest.NE.'no') THEN
        WRITE (10, '('' '')')
        WRITE (10, '(''Number of ancestors requested for analysis by the user: '',I10)') Write_NFF
        IF (NF.LT.NFF) WRITE (10, '(''lower than total number to be analysed: '',I10)') NF
      END IF

      WRITE (10, '('' '')')
      WRITE (10, '(''NB of alleles in segregation: '',I10)') NALL
      WRITE (10, '('' '')')
      WRITE (10, '(''Total Number of founders analysed for contributions: '',I10)') NFF
      WRITE (10, '('' '')')
      WRITE (10,'(''Elapsed CPU time: '',I3.2,'':'',I2.2,'':'',I2.2)') Hour, Minute, Sec
      WRITE (10, '('' '')')
      WRITE (10, '(''End Date: '',A)') End_Date_Time
      WRITE (10, '(''**********************************'')')

      CLOSE (10)

      OPEN (10, FILE='fort.10', STATUS='OLD', IOSTAT=IOS)
      IF (IOS .EQ. 0) CLOSE (10, STATUS='DELETE')

      CALL Press_Enter()

      END

      SUBROUTINE Press_Enter()
      PRINT *, ''
      PRINT *, 'Press Enter (Return) to complete the program ...'
      READ(*,'(A)') Str
      PRINT *, ''
      STOP
      END SUBROUTINE

      SUBROUTINE Press_Enter_Memory()
      PRINT *, ''
      PRINT *, '!!! Memory is small !!!'
      PRINT *, ''
      PRINT *, ''
      PRINT *, 'Press Enter (Return) to complete the program ...'
      READ(*,'(A)') Str
      PRINT *, ''
      STOP
      END SUBROUTINE
!      
!         subroutine random_number(x)
!	  real x,g05caf
!	  external g05caf
!	  x=g05caf()
!	  return
!	  end

